<template>
  <div class="efficiency_list" style="width: 100%;height: 100%;">
    <div :bordered="false" style="height: 100%">
      <!-- 查询区域 -->
      <div class="table-page-search-wrapper">
        <a-form layout="inline" @keyup.enter.native="searchQuery">
          <a-row :gutter="24">
            <a-col :md="6" :sm="6">
              <a-form-item label="设备类型">
                <a-select
                  :value="queryParams.equipmentType"
                  mode="multiple"
                  placeholder="请选择设备类型"
                  allow-clear
                  :maxTagCount="1"
                  @change="selectChange($event,'equipmentType')"
                >
                  <a-select-option v-for="(item,index) in equipmentTypeList" :value="item.value" :key="index">
                    {{ item.label }}
                  </a-select-option>
                </a-select>
              </a-form-item>
            </a-col>

            <a-col :md="6" :sm="6">
              <a-form-item label="驱动类型">
                <a-select
                  :value="queryParams.driveType"
                  mode="multiple"
                  placeholder="请选择驱动类型"
                  allow-clear
                  :maxTagCount="1"
                  @change="selectChange($event,'driveType')"
                >
                  <a-select-option v-for="(item,index) in driveTypeList" :value="item.value" :key="index">
                    {{ item.label }}
                  </a-select-option>
                </a-select>
              </a-form-item>
            </a-col>

            <a-col :md="6" :sm="6">
              <a-form-item label="时间">
                <a-range-picker @change="dateParamChange" :disabledDate="disabledDate" format="YYYYMMDD"
                                v-model="dates" :allowClear="false"/>
              </a-form-item>
            </a-col>

            <a-col :md="3" :sm="3" :xs="3">
              <a-space>
                <a-button type="primary" @click="searchQuery" icon="search">查询</a-button>
                <a-button type="primary" @click="searchReset" icon="reload">重置</a-button>
              </a-space>
            </a-col>

            <a-col :md="2" :sm="2">
              <a
                @click="toggleSearchStatus=!toggleSearchStatus"
                @selectstart="$event.preventDefault()"
                style="display: inline-block;height: 32px;line-height: 32px"
              >
                {{ toggleSearchStatus ? '收起' : '展开' }}
                <a-icon :type="toggleSearchStatus ? 'up' : 'down'"/>
              </a>
            </a-col>
          </a-row>

          <a-row :gutter="24" v-if="toggleSearchStatus">
            <a-col :md="6" :sm="6" :xs="6">
              <a-form-item label="设备级别">
                <a-select
                  :value="queryParams.deviceLevel"
                  mode="multiple"
                  placeholder="请选择设备级别"
                  allow-clear
                  :maxTagCount="1"
                  @change="selectChange($event,'deviceLevel')"
                >
                  <a-select-option v-for="(item,index) in device_level_list" :value="item.value" :key="index">
                    {{ item.label }}
                  </a-select-option>
                </a-select>
              </a-form-item>
            </a-col>

            <a-col :md="5" :sm="5" :xs="5">
              <a-form-item label="重要程度">
                <a-select
                  v-model="queryParam.deviceImportanceLevel"
                  placeholder="请选择设备重要程度"
                  allow-clear
                >
                  <a-select-option v-for="(item,index) in device_importance_level_list" :value="item.value"
                                   :key="index">
                    {{ item.label }}
                  </a-select-option>
                </a-select>
              </a-form-item>
            </a-col>
          </a-row>
        </a-form>
      </div>

      <!--表格及图表区域-->
      <div id="DeviceList">
        <!--表格区域-->
        <a-spin :spinning="loading">
          <div id="Efficiency" class="container" style="margin-bottom: 15px;">
            <div class="table2">
              <table class="dataContent table" border="1" cellspacing="0" cellpadding="0"
                     style="white-space: nowrap;text-align: center;">
                <thead>
                <tr class="thead fixed equipname">
                  <th class="thgu dong1 name" rowspan="2" style="min-width: 150px; max-width: 150px;width: 150px;">
                    设备编号
                  </th>
                  <th class="thgu dong2 name" rowspan="2" style="min-width: 162px; max-width: 162px;width: 162px;">
                    设备名称
                  </th>
                  <th class="thgu dong3 name" rowspan="2" style="min-width: 100px; max-width: 100px;width: 100px;">
                    设备类型
                  </th>
                  <template v-for="(tableHead, index) in tableHeads">
                    <th class="timeth" :colspan="checkedList.length">{{ tableHead }}</th>
                  </template>
                </tr>
                </thead>
                <tbody>
                <tr class="mathData" v-for="(item, index) in dataList" @click="TableDraw(index,item)">
                  <td class="tdgu  kaitou">{{ item.equipmentId }}</td>
                  <td class="tdgu1 kaitou">{{ item.equipmentName }}</td>
                  <td class="tdgu2 kaitou">{{ item.equipmentType }}</td>
                  <template v-for="(tableHead, index) in item.dataList">
                    <td :style="{background:tableHead.color }">{{ tableHead.utilizationRate | numFilter }}</td>
                  </template>
                </tr>
                </tbody>
              </table>
            </div>
          </div>
        </a-spin>
        <!--图标区域-->
        <div id="openRateTrendChart"></div>
      </div>
    </div>
  </div>
</template>

<script>
import moment from 'moment'
import { getAction } from '@/api/manage'
import $ from 'jquery'
import '@/components/table2excel/table2excel'
import { ajaxGetDictItems, getDictItemsFromCache } from '@/api/api'
import api from '@api/mdc'

export default {
  name: 'openRateTrendAnalysisMain',
  components: {},
  data() {
    return {
      dataSource: [],
      /* table加载状态 */
      loading: false,
      typeTree: '',
      typeParent: 1,
      typeEquipment: 1,
      TreeIDOne: 1,
      TreeIDTwo: 2,
      deviceTypeDict: '',
      dates: [],
      equipmentTypeList: [],
      driveTypeList: [],
      device_level_list: [],
      device_importance_level_list: [],
      identifying: [],
      queryParam: {},
      queryParams: {
        equipmentType: [],
        driveType: [],
        deviceLevel: []
      },
      queryParamEquip: {},
      queryParamPeople: {},
      efficiencyOptions: [
        { label: '利用率', value: 'lyl' },
        { label: '开机率', value: 'kjl' },
        { label: '开机时间', value: 'kjsj' },
        { label: '加工时间', value: 'jgsj' },
        { label: '待机时间', value: 'djsj' },
        { label: '关机时间', value: 'gjsj' }
      ],
      checkedList: ['lyl'],
      dataList: [],
      url: {
        efficiencyList: '/mdc/efficiencyReport/utilizationRateTrendAnalyze',
        listByType: '/mdc/MdcUtilizationRate/getByType',
        queryEquipmentType: '/mdc/mdcEquipmentType/queryEquipmentType'
      },
      tableHeads: [],
      XData: [0],
      YData: [0],
      isLazyRequest: '',// 是否开启懒加载
      requestAlldataSize: 8,// 总共请求的列表项数目
      everyRequestDataSize: 15,// 每次请求的列表项数目
      toggleSearchStatus: false
    }
  },
  props: { nodeTree: '', Type: '', nodePeople: '' },
  /**
   * 生命周期 挂载前
   * */
  created() {
    this.searchReset()
    this.queryGroup()
    this.getDriveTypeByApi()
    this.initDictData('device_level')
    this.initDictData('device_importance_level')
  },
  mounted() {
    this.tableScroll = document.querySelector('.table2')
    window.addEventListener('resize', this.handleWindowResize)
  },
  beforeDestroy() {
    this.tableScroll.removeEventListener('scroll', this.tableScrollX)
    window.removeEventListener('resize', this.handleWindowResize)
  },
  watch: {
    Type(valmath) {
      this.dataList = []
      this.queryParam.typeTree = valmath
    },
    nodeTree(val) { //监听currSelected 变化，将变化后的数值传递给 getCurrSelected 事件
      if (JSON.stringify(val) != '{}') {
        if (val.equipmentId != null) {
          this.queryParamEquip.parentId = ''
          this.queryParamEquip.equipmentId = val.equipmentId
        } else {
          this.queryParamEquip.parentId = val.key
          this.queryParamEquip.equipmentId = ''
        }
        /*这里你是监听到树得变化 是不是请求一次*/
        this.searchQuery()
      }
    },
    nodePeople(val) {
      if (JSON.stringify(val) != '{}') {
        if (val.equipmentId != null) {
          this.queryParamPeople.parentId = val.equipmentId
          this.queryParamPeople.equipmentId = ''
        } else {
          this.queryParamPeople.parentId = val.key
          this.queryParamPeople.equipmentId = ''
        }
        this.searchQuery()
      }
    }
  },
  filters: {
    numFilter(value) {
      if (value) {
        return parseFloat((value * 100).toFixed(2))
      } else {
        return '0'
      }
    }
  },
  methods: {
    queryGroup() {
      getAction(this.url.queryEquipmentType).then(res => {
        if (res.success) {
          this.equipmentTypeList = res.result.map(item => {
            return {
              label: item.equipmentTypeName,
              value: item.equipmentTypeName
            }
          })
        } else {
          this.$notification.warning({
            message: '消息',
            description: res.message
          })
        }
      })
    },

    // 调用接口获取控制系统类型
    getDriveTypeByApi() {
      api.getDriveTypeApi().then((res) => {
        if (res.success) this.driveTypeList = res.result
      })
    },

    initDictData(dictCode) {
      // //优先从缓存中读取字典配置
      if (getDictItemsFromCache(dictCode)) {
        this[dictCode + '_list'] = getDictItemsFromCache(dictCode)
        return
      }
      //根据字典Code, 初始化字典数组
      ajaxGetDictItems(dictCode, null).then((res) => {
        if (res.success) {
          this[dictCode + '_list'] = res.result
          return
        }
      })
    },

    loadData() {
      this.loading = true
      this.tableHeads = []
      this.dataList = []
      // 若日期选择器设置的查询请求数量超过15条则开启懒加载
      if (this.requestAlldataSize > 15) {
        // 开启懒加载后将结束日期设置为开始日期之后的14天，即先查询15天内的数据
        this.queryParam.endDate = moment(this.queryParam.startDate).add(this.everyRequestDataSize - 1, 'days').format('YYYYMMDD')
        this.isLazyRequest = true
        this.tableScroll.removeEventListener('scroll', this.tableScrollX) // 为避免滚动条有滚动距离后后点击查询按钮触发滚动事件，因此在触发事件前移除事件
      }
      getAction(this.url.efficiencyList, this.queryParam).then(res => {
        if (res.success) {
          this.tableHeads = res.result.dates
          this.dataList = res.result.mdcEfficiencyList
          this.draw()
          this.tableScroll.addEventListener('scroll', this.tableScrollX)
        }
      }).finally(() => {
        this.loading = false
      })
    },

    TableDraw(key, val) {
      this.XData = [0]
      this.YData[0]
      var Xarr = []
      var Yarr = []
      for (var i = 0; i < val.dataList.length; i++) {
        Xarr.push(val.dataList[i].theDate)
        Yarr.push(this.numBerTwo(val.dataList[i].utilizationRate))
      }
      this.XData = Xarr
      this.YData = Yarr

      this.draw()
    },

    searchQuery() {
      // 因触底刷新会修改参数时间值，故在此用时间选择器的时间值对参数时间值再次赋值，若不这样做则参数时间值则会与时间选择器上的时间值不一致
      this.queryParam.startDate = moment(this.dates[0]).format('YYYYMMDD')
      this.queryParam.endDate = moment(this.dates[1]).format('YYYYMMDD')

      if (this.queryParam.typeTree == '1') {
        this.queryParam.parentId = this.queryParamEquip.parentId
        this.queryParam.equipmentId = this.queryParamEquip.equipmentId
      } else {
        this.queryParam.parentId = this.queryParamPeople.parentId
        this.queryParam.equipmentId = ''
      }
      this.requestAlldataSize = moment.duration(moment(this.queryParam.endDate) - moment(this.queryParam.startDate), 'millisecond').asDays() + 1

      Object.keys(this.queryParams).forEach(item => {
        if (Array.isArray(this.queryParams[item])) {
          if (this.queryParams[item].length !== 0) this.queryParam[item] = this.queryParams[item].join()
          else delete this.queryParam[item]
        }
      })
      this.loadData()
    },

    searchReset() {
      this.queryParams = {
        equipmentType: [],
        driveType: [],
        deviceLevel: []
      }
      this.queryParam = {}
      this.dates = [moment().subtract('days', 8), moment().subtract('days', 1)]
      this.queryParam.startDate = moment(this.dates[0]).format('YYYYMMDD')
      this.queryParam.endDate = moment(this.dates[1]).format('YYYYMMDD')
      this.loadData()
    },

    draw() {
      this.openRateTrendAnalysisChart = this.$echarts.init(document.getElementById('openRateTrendChart'), 'macarons')
      let openRateTrendChartOptions = {
        title: {
          text: '利用率走势分析',
          x: 'center',
          textStyle: {
            fontSize: 18,
            fontWeight: 'bolder',
            color: '#151414'
          }
        },
        tooltip: {
          trigger: 'axis'
        },
        toolbox: {
          show: true,
          feature: {
            mark: { show: true },
            dataView: { show: true, readOnly: false },
            magicType: { show: true, type: ['line', 'bar'] },
            restore: { show: true },
            saveAsImage: { show: true }
          }
        },
        calculable: true,
        xAxis: [
          {
            type: 'category',
            data: this.XData
          }
        ],
        yAxis: [
          {
            type: 'value'
          }
        ],
        series: [
          {
            name: '利用率',
            type: 'line',
            data: this.YData,
            markPoint: {
              data: [
                { type: 'max', name: '最大值', symbolSize: 70 },
                { type: 'min', name: '最小值', symbolSize: 70 }
              ]
            },
            markLine: {
              data: [
                { type: 'average', name: '平均值' }
              ]
            }
          }
        ]
      }
      this.openRateTrendAnalysisChart.setOption(openRateTrendChartOptions, true)
    },

    selectChange(value, key) {
      this.queryParams[key] = value
    },

    numBerTwo(value) {
      if (value) {
        return parseFloat((value * 100).toFixed(2))
      } else {
        return '0'
      }
    },

    disabledDate(current) {
      //Can not slect days before today and today
      return current && current > moment().subtract('days', 1)
    },

    dateParamChange(v1, v2) {
      this.queryParam.startDate = v2[0]
      this.queryParam.endDate = v2[1]
      this.dates = [v1[0], v1[1]]
    },

    /**
     * 滚动条触底刷新表格数据
     */
    tableScrollX() {
      // 当滚动条触底且懒加载开启时触发滚动条触底刷新
      if (Math.ceil(this.tableScroll.scrollLeft + this.tableScroll.clientWidth) + 1 >= this.tableScroll.scrollWidth && this.isLazyRequest) {
        this.loading = true
        this.queryParam.startDate = moment(this.queryParam.endDate).add(1, 'days').format('YYYYMMDD')
        // 判断设置懒加载后的结束时间是否超过时间选择器中设置的结束时间
        if (moment(this.queryParam.endDate).add(this.everyRequestDataSize - 1, 'days').format('YYYYMMDD') - moment(this.dates[1]).format('YYYYMMDD') < 0) {
          //再次查询后15天内的数据
          this.queryParam.endDate = moment(this.queryParam.endDate).add(this.everyRequestDataSize, 'days').format('YYYYMMDD')
        } else {
          // 如果不小于时间选择器中设置的时间则设置请求结束日期为时间选择器中设置的结束时间并且关闭懒加载（意味着最后一次触底刷新）
          this.queryParam.endDate = moment(this.dates[1]).format('YYYYMMDD')
          this.isLazyRequest = false
        }
        getAction(this.url.efficiencyList, this.queryParam).then(res => {
          if (res.success) {
            this.tableHeads = [...this.tableHeads, ...res.result.dates]
            this.dataList.forEach(item1 => {
              res.result.mdcEfficiencyList.forEach(item2 => {
                if (item1.equipmentId === item2.equipmentId) {
                  item1.dataList = [...item1.dataList, ...item2.dataList]
                }
              })
            })
            this.draw()
          }
        }).finally(() => {
          this.loading = false
        })
      }
    },

    /**
     * 当浏览器可视窗口尺寸发生改变时触发
     */
    handleWindowResize() {
      if (this.openRateTrendAnalysisChart) this.openRateTrendAnalysisChart.resize()
    }
  }
}
</script>
<style scoped>
.dataContent {
  white-space: nowrap;
  /*margin: 0;*/
  /*border: none;*/
  border-collapse: separate;
  border-spacing: 0;
  /*table-layout: fixed;*/
  border: 1px solid #ccc;
  /*border: 1px solid #ccc;*/
  width: 100%;
  text-align: center;
}

.dataContent .thead th {
  background-color: #fafafa;
  text-align: center;
  height: 30px;
  padding: 5px;
}

.dataContent tr td {
  height: 35px;
}

.dataContent .mathData td {
  padding: 10px;
  cursor: pointer;
}

.dataContent .mathData:hover td {
  background-color: #e6f7ff;
}

.table2 {
  width: 100%;
  height: 100%;
  overflow: auto;
}

.table2 thead tr th:first-child,
.table tbody tr .tdgu {
  position: sticky;
  left: 0;
  z-index: 1;
}

.table tbody tr .kaitou {
  z-index: 1;
  background-color: white;
}

.table tbody tr .tdgu1 {
  position: sticky;
  left: 150px;
  z-index: 2;
}

.table tbody tr .tdgu2 {
  position: sticky;
  left: 312px;
  z-index: 2;
}

.table2 thead tr .timeth,
.table2 thead tr .thgu {
  position: sticky;
  top: 0;
  z-index: 3;
}

.table2 thead .gudingth th {
  position: sticky;
  top: 32px;
  z-index: 6 !important;
}

.table2 thead .equipname .name {
  z-index: 3;
}

.table2 thead .equipname .dong1 {
  z-index: 4;
  left: 0;
}

.table2 thead .equipname .dong2 {
  z-index: 5;
  left: 150px;
}

.table2 thead .equipname .dong3 {
  z-index: 5;
  left: 312px;
}

.efficiency_list #DeviceList {
  height: 90% !important;
}

#Efficiency {
  overflow: hidden;
}

#openRateTrendChart {
  width: 100%;
  height: 60%;
}

@media screen and (min-width: 1920px) {
  #Efficiency {
    height: 337px !important;
  }
}

@media screen and (min-width: 1680px) and (max-width: 1920px) {
  #Efficiency {
    height: 337px !important;
  }
}

@media screen and (min-width: 1400px) and (max-width: 1680px) {
  #Efficiency {
    height: 337px !important;
  }
}

@media screen and (min-width: 1280px) and (max-width: 1400px) {
  #Efficiency {
    height: 200px !important;
  }
}

@media screen and (max-width: 1280px) {
  #Efficiency {
    height: 200px !important;
  }
}


</style>